-- PBARS_HEALTH/MANA left, bottom, width, height, r, g, b, a, f
-- PBARS_EXT 1..10 reserved, 11..20 - player1, 21..30 - player2, player3, player4
-- [1] width, [2] height, [3] class height, [4] show level; [11] left, [12] bottom

local helpText = {
"|cFFffff00PlayerBars short help and command list:|r",
"|cFFffff00For full documentation please visit|r",
"|cFF00ffffhttp://www.curse.com/addons/wow/playerbars|r",
"Use Shift + Mouse Left Click to drag elements",
"|cFF00ffff/pbars|r - toggles all elements",
"|cFF00ffff/pbars hw <number>|r - sets health bar width",
"|cFF00ffff/pbars hh <number>|r - sets health bar height",
"|cFF00ffff/pbars hc <number> <number> <number> <number>|r",
"- sets health bar rgba color",
"- use |cFF00ffffmw|r, |cFF00ffffmh|r, |cFF00ffffmc|r instead of |cFF00ffffhw|r, |cFF00ffffhh|r, |cFF00ffffhc|r to change power bar",
"|cFFffff00Group commands:|r",
"|cFF00ffff/pbars g|r - toggles group swatches",
"|cFF00ffff/pbars gw <number>|r - sets swatch width",
"|cFF00ffff/pbars gh <number>|r - sets swatch height",
"|cFFffff00Block commands:|r",
"|cFF00ffff/pbars b|r - justifies both bars and puts them together",
"|cFF00ffff/pbars bgh <number>|r - lines up swatches in a row separated by value",
"|cFF00ffff/pbars bgv <number>|r - lines up swatches in a column separated by value"
}


local hWidth, mWidth, hHeight, mHeight, timerInt, hr, hg, hb, ha, mr, mg, mb, ma

local hframe, mframe

local timer = 0.0
local hRatio = 1.0
local mRatio = 1.0
local rleft, rbottom, rwidth, rheight
local htexture, mtexture

local playerName = UnitName("player")

local xRGBA = {
	["high"] = {0, 1, 0, 0.5},
	["mid"] =  {1, 1, 0, 0.5},
	["low"] =  {1, 1, 0, 0.5},
	["crit"] = {1, 0, 0, 0.5},
	["zero"] = {0, 0, 0, 0.5},
	["off"] =  {0.6, 0.6, 0.6, 0.3}
}
-- swatches, frame ref, text ref, class frame ref, party index
xRefs = {{nil,nil,nil}, {nil,nil,nil}, {nil,nil,nil}, {nil,nil,nil}}

local className = {"WR", "PL", "HT", "RG", "PR", "DK", "SH", "MG", "LK", "MN", "DR"}  
local roleName = {["DAMAGER"] = "D", ["HEALER"] = "H", ["NONE"] = "X", ["TANK"] = "T"}

local function GetGroupInfo()
    local unit, classInd, role, name, hRatio
	for i = 1, 4 do
		unit = "party" .. i
		if UnitExists(unit) then
			name = UnitName(unit)
			if name ~= playerName then
				-- DAMAGER, HEALER, NONE, TANK
				role = UnitGroupRolesAssigned(unit)
				_, _, classInd = UnitClass(unit)
				hRatio = UnitHealth(name) / UnitHealthMax(name)
				
				if not UnitIsConnected(name) or UnitHealth(name) < 2 then hRatio = 0 end
				
				if hRatio > 0.75 then
					xRefs[i][2]:SetTexture(unpack(xRGBA["high"]))
				elseif hRatio > 0.5 then
					xRefs[i][2]:SetTexture(unpack(xRGBA["mid"]))
				elseif hRatio > 0.25 then
					xRefs[i][2]:SetTexture(unpack(xRGBA["low"]))
				elseif hRatio > 0 then
					xRefs[i][2]:SetTexture(unpack(xRGBA["crit"]))
				else
					xRefs[i][2]:SetTexture(unpack(xRGBA["zero"]))
				end
				
				xRefs[i][1].text:SetText(roleName[role])
				if classInd ~= nill then
					xRefs[i][3].text:SetText(className[classInd] .. "-" .. UnitLevel(unit))
				else
					xRefs[i][3].text:SetText("??" .. "-" .. UnitLevel(unit))
				end
			end
		else
			xRefs[i][2]:SetTexture(unpack(xRGBA["zero"]))
			xRefs[i][1].text:SetText("-")
			xRefs[i][3].text:SetText("-")
		end
	end
end

local function onUpdate(self,elapsed)
	local tmp
	timer = timer + elapsed
	if timer > timerInt then
		hRatio = UnitHealth(playerName) / UnitHealthMax(playerName)
		mRatio = UnitPower(playerName) / UnitPowerMax(playerName)
		
		rleft, rbottom, rwidth, rheight = hframe:GetBoundsRect()
		hframe:ClearAllPoints()
		hframe:SetPoint("TOPLEFT", WorldFrame, "BOTTOMLEFT", rleft, rbottom + hHeight)
		tmp = hWidth * hRatio
		if tmp == 0 then
			tmp = 1
		end
		hframe:SetWidth(tmp)

		rleft, rbottom, rwidth, rheight = mframe:GetBoundsRect()
		mframe:ClearAllPoints()
		mframe:SetPoint("TOPLEFT", WorldFrame, "BOTTOMLEFT", rleft, rbottom + mHeight)
		tmp = mWidth * mRatio
		if tmp == 0 then
			tmp = 1
		end
		mframe:SetWidth(tmp)

		--[[attack state
		local r, g, b = GetThreatStatusColor(UnitThreatSituation("player"))
		local a = 1
		if r == g and g == b then
			a = 0.5
		end
		atexture:SetTexture(r, g, b, a)]]--
		
		-- group state
		if xRefs[1][1]:IsVisible() then
			if IsInGroup() then 
				GetGroupInfo()
			else
				-- reset empty values
				for i = 1, 4 do
					xRefs[i][2]:SetTexture(unpack(xRGBA["off"]))
					xRefs[i][1].text:SetText(" ")
					xRefs[i][3].text:SetText(" ")
				end
			end
		end
		
		timer = 0
	end
end

local function onMouseDown(self, button)
	if button == "LeftButton" and IsShiftKeyDown() and not self.isMoving then
		self:StartMoving();
		self.isMoving = true;
	end
end

local function onMouseUp(self, button)
	if button == "LeftButton" then
		if self.isMoving then
			self:StopMovingOrSizing();
			self.isMoving = false;
			-- left, bottom
			PBARS_HEALTH[1], PBARS_HEALTH[2] = hframe:GetBoundsRect()
			PBARS_MANA[1],   PBARS_MANA[2]   = mframe:GetBoundsRect()
			-- swatches left, bottom, inc frClass
			for i = 1, 4 do
				PBARS_EXT[10 * i + 1], PBARS_EXT[10 * i + 2] = xRefs[i][1]:GetBoundsRect()
			end
		end
	end
end

local function setFrameProps(frRef)
	frRef:SetMovable(true)
	frRef:EnableMouse(true)
	frRef:SetScript("OnMouseDown", onMouseDown)
	frRef:SetScript("OnMouseUp", onMouseUp)
end

local function createFrame(fname, x, y, w, h, RGBA)
	local frRef, texRef
	frRef = CreateFrame("Frame", fname, UIParent)
	frRef:SetClampedToScreen(true)
	frRef:SetSize(w, h) 
	frRef:SetPoint("TOPLEFT", WorldFrame, "BOTTOMLEFT", x, y)
	texRef = frRef:CreateTexture()
	texRef:SetAllPoints()
	local hr, hg, hb, ha = select(1, unpack(RGBA))
	texRef:SetTexture(hr, hg, hb, ha) 
	frRef.background = texRef
	
	setFrameProps(frRef)
	
	return frRef, texRef
end

local function createSwatch(fname, x, y, w, h, txt, txt2, RGBA)
	
	local frRef, texRef = createFrame(fname, x, y, w, h, RGBA)
	local frClass, frClassTex

	if txt ~= nil then
		frRef.text = frRef:CreateFontString(nil,"ARTWORK")
		frRef.text:SetFontObject("GameFontHighlightLarge")
		frRef.text:SetTextColor(1, 1, 1, 0.5)
		frRef.text:SetText(txt)
		frRef.text:SetAllPoints()
	end
	
	frClass = CreateFrame("Frame", "ClassFrame", frRef)
	frClass:SetSize(w, PBARS_EXT[3]) 
	frClass:SetPoint("BOTTOMLEFT", frRef, "BOTTOMLEFT", 0, -PBARS_EXT[3])
	frClassTex = frClass:CreateTexture()
	frClassTex:SetAllPoints()
	frClassTex:SetTexture(0, 0, 0, 0.5) 
	frClass.background = frClassTex
	
	if txt ~= nil then
		frRef.text = frRef:CreateFontString(nil,"ARTWORK")
		frRef.text:SetFontObject("GameFontHighlightLarge")
		frRef.text:SetTextColor(1, 1, 1, 0.5)
		frRef.text:SetText(txt)
		frRef.text:SetAllPoints()
	end
	
	if txt2 ~= nil then
		frClass.text = frClass:CreateFontString(nil,"ARTWORK")
		frClass.text:SetFontObject("Tooltip_Small")
		frClass.text:SetTextColor(1, 1, 0, 0.5)
		frClass.text:SetText(txt2)
		frClass.text:SetAllPoints()
	end
	
	setFrameProps(frRef)
	
	return frRef, texRef, frClass
end

local eframe = CreateFrame("Frame", "EventFrame", UIParent)
eframe:RegisterEvent("ADDON_LOADED")
function eframe:OnEvent(event, arg)
if event == "ADDON_LOADED" and arg == "PlayerBars" then

	if PBARS_HEALTH == nil then	PBARS_HEALTH = {} end
	if PBARS_MANA   == nil then	PBARS_MANA   = {} end
	if PBARS_EXT    == nil then PBARS_EXT    = {} end
	
	-- PBARS_HEALTH/MANA left, bottom, width, height, r, g, b, a, f
	hWidth = PBARS_HEALTH[3] or 265
	mWidth = PBARS_MANA[3] or 265
	hHeight = PBARS_HEALTH[4] or 6
	mHeight = PBARS_MANA[4] or 6
	timerInt = PBARS_HEALTH[9] or 0.2
	hr = PBARS_HEALTH[5] or 0
	hg = PBARS_HEALTH[6] or 0.7
	hb = PBARS_HEALTH[7] or 0
	ha = PBARS_HEALTH[8] or 1
	
	PBARS_EXT[1] = PBARS_EXT[1] or 40
	PBARS_EXT[2] = PBARS_EXT[2] or 20
	PBARS_EXT[3] = PBARS_EXT[3] or 12
	PBARS_EXT[4] = PBARS_EXT[4] or true
	
	-- mana bar colors
	local pTypes = {
		{"MANA", 0,0,0.7,1}, -- 1
		{"RAGE", 0.7,0,0,1}, -- 2
		{"FOCUS", 0.6,0.4,0.3,1}, -- 3
		{"ENERGY", 0.7,0.7,0,1}, -- 4
		{"COMBO_POINTS", 0.5,0.5,0.5,1}, -- 5
		{"RUNES", 0.5,0.5,0.5,1}, -- 6
		{"RUNIC_POWER", 0.5,0.5,0.5,1}, -- 7
		{"SOUL_SHARDS", 0.5,0.5,0.5,1}, -- 8 
		{"ECLIPSE", 0.5,0.5,0.5,1}, -- 9
		{"HOLY_POWER", 0.5,0.5,0.5,1}, -- 10
	}
	local pt = pTypes[UnitPowerType("player")+1]

	mr = PBARS_MANA[5] or pt[2]
	mg = PBARS_MANA[6] or pt[3]
	mb = PBARS_MANA[7] or pt[4]
	ma = PBARS_MANA[8] or pt[5]
	
	local centerX = GetScreenWidth()/2
	local centerY = GetScreenHeight()/2
	
	local startHX = PBARS_HEALTH[1] or centerX
	local startMX = PBARS_MANA[1]   or centerX
	local startHY = centerY
	local startMY = centerY - hHeight
	
	local startXX = {}
	local startXY = {}
	for i = 1, 4 do
		startXX[i] = PBARS_EXT[10*i + 1] or centerX + 40 + (PBARS_EXT[1] + 10) * (i - 1)
		startXY[i] = centerY - 20
	end
	
	if PBARS_HEALTH[2] then startHY  = PBARS_HEALTH[2] + hHeight else PBARS_HEALTH[2] = startHY  - hHeight end
	if PBARS_MANA[2]   then startMY  = PBARS_MANA[2]   + mHeight else PBARS_MANA[2]   = startMY  - mHeight end
	
	for i = 1, 4 do
		if PBARS_EXT[10*i + 2] then
			startXY[i] = PBARS_EXT[10*i + 2] + PBARS_EXT[2] + PBARS_EXT[3]
		else 
			PBARS_EXT[10*i + 2] = startXY[i] - PBARS_EXT[2] - PBARS_EXT[3]
		end
	end
	
	hframe, htexture = createFrame("HealthFrame", startHX, startHY, hWidth, hHeight, {hr, hg, hb, ha})
	mframe, mtexture = createFrame("ManaFrame",   startMX, startMY, mWidth, mHeight, {mr, mg, mb, ma})
	
	for i = 1, 4 do
		xRefs[i][1], xRefs[i][2], xRefs[i][3] = 
			createSwatch("Swatch"..i, startXX[i], startXY[i], PBARS_EXT[1], PBARS_EXT[2], " ", " ", xRGBA["off"])
		xRefs[i][4] = i
	end

	hframe:SetScript("OnUpdate", onUpdate)
	
	print(format("|cFFffff00PlayerBars v0.3.3 loaded. Type /pbars -h to see command list.|r"))

end
end
eframe:SetScript("OnEvent", eframe.OnEvent)

local function updateSwatches()
	for i = 1, 4 do
		xRefs[i][1]:SetSize(PBARS_EXT[1], PBARS_EXT[2])
		xRefs[i][3]:SetSize(PBARS_EXT[1], PBARS_EXT[3])
	end
end

SLASH_PLAYERBARS1 = "/pbars"
function SlashCmdList.PLAYERBARS(msg, editbox)
	local hupdate = false
	local mupdate = false
	local left, bottom
	local cmd, args = strsplit(" ", strlower(msg), 2)
	local r, g, b, a
	if cmd == "" then
		if hframe:IsVisible() then
			hframe:Hide()
			mframe:Hide()
			for i = 1, 4 do
				xRefs[i][1]:Hide()
			end
			hframe:SetScript("OnUpdate", nil)
		else
			hframe:Show()
			mframe:Show()
			for i = 1, 4 do
				xRefs[i][1]:Show()
			end
			hframe:SetScript("OnUpdate", onUpdate)
		end
	
	elseif cmd == "f" and tonumber(args) > 0 then
		timerInt = tonumber(args)
		PBARS_HEALTH[9] = timerInt
	
	elseif cmd == "hw" and tonumber(args) > 0 then
		hWidth = tonumber(args)
		PBARS_HEALTH[3] = hWidth
		hupdate = true
	elseif cmd == "hh" and tonumber(args) > 0 then
		hHeight = tonumber(args)
		PBARS_HEALTH[4] = hHeight
		hupdate = true	
	elseif cmd == "hc" and args ~= "" then
		r, g, b, a = strsplit(" ", strlower(args), 4)
		htexture:SetTexture(r, g, b, a)
		hframe.background = htexture
		PBARS_HEALTH[5] = r
		PBARS_HEALTH[6] = g
		PBARS_HEALTH[7] = b
		PBARS_HEALTH[8] = a
		
	elseif cmd == "mw" and tonumber(args) > 0 then
		mWidth = tonumber(args)
		PBARS_MANA[3] = mWidth
		mupdate = true
	elseif cmd == "mh" and tonumber(args) > 0 then
		mHeight = tonumber(args)
		PBARS_MANA[4] = mHeight
		mupdate = true	
	elseif cmd == "mc" and args ~= "" then
		r, g, b, a = strsplit(" ", strlower(args), 4)
		mtexture:SetTexture(r, g, b, a)
		mframe.background = mtexture
		PBARS_MANA[5] = r
		PBARS_MANA[6] = g
		PBARS_MANA[7] = b
		PBARS_MANA[8] = a
	
	elseif cmd == "b" then
		mWidth = hWidth
		PBARS_MANA[3] = mWidth
		left, bottom = hframe:GetBoundsRect()
		mframe:SetPoint("TOPLEFT", WorldFrame, "BOTTOMLEFT", left, bottom)
		PBARS_MANA[1], PBARS_MANA[2] = mframe:GetBoundsRect()
		mupdate = true
	elseif cmd == "bgh" and args ~= "" then
		xRefs[1][1]:Show()
		left, bottom = xRefs[1][1]:GetBoundsRect()
		for i = 2, 4 do
			xRefs[i][1]:SetPoint("TOPLEFT", WorldFrame, "BOTTOMLEFT", 
								 left + (i - 1)*(PBARS_EXT[1] + args), 
								 bottom + PBARS_EXT[2] + PBARS_EXT[3])
			xRefs[i][1]:Show()
			PBARS_EXT[10 * i + 1], PBARS_EXT[10 * i + 2] = xRefs[i][1]:GetBoundsRect()
		end
	elseif cmd == "bgv" and args ~= "" then
		xRefs[1][1]:Show()
		left, bottom = xRefs[1][1]:GetBoundsRect()
		for i = 2, 4 do
			xRefs[i][1]:SetPoint("TOPLEFT", WorldFrame, "BOTTOMLEFT", 
								 left, 
								 bottom + PBARS_EXT[2] + PBARS_EXT[3] - (i - 1)*(PBARS_EXT[2] + PBARS_EXT[3] + args))
			xRefs[i][1]:Show()
			PBARS_EXT[10 * i + 1], PBARS_EXT[10 * i + 2] = xRefs[i][1]:GetBoundsRect()
		end
			
	elseif cmd == "gw" and args ~= "" then
		PBARS_EXT[1] = args
		updateSwatches()
	elseif cmd == "gh" and args ~= "" then
		PBARS_EXT[2] = args
		updateSwatches()
	elseif cmd == "g" then
		if xRefs[1][1]:IsVisible() then
			for i = 1, 4 do
				xRefs[i][1]:Hide()
			end
		else
			for i = 1, 4 do
				xRefs[i][1]:Show()
			end
		end
		
	elseif cmd == "-help" or cmd == "-h" or cmd == "?" or cmd == "/?" then
		for _, v in pairs(helpText) do
			print(format("%s", v))
		end
	end
	
	if hupdate then
		hframe:SetSize(hWidth * hRatio, hHeight)
	end
	if mupdate then
		mframe:SetSize(mWidth * mRatio, mHeight)
	end
end



